@@ -5,17 +5,17 @@ import os
 import sys
 
 
-def grandchild():
+def grandchild() -> None:
     sys.stdout.write("grandchild started")
     sys.stdout.flush()
     sys.stdin.read()
 
 
-def main():
+def main() -> None:
     if sys.argv[1] == "child":
         if sys.argv[2] == "windows":
-            import win32api as api  # type: ignore[import]
-            import win32process as proc  # type: ignore[import]
+            import win32api as api
+            import win32process as proc
 
             info = proc.STARTUPINFO()
             info.hStdInput = api.GetStdHandle(api.STD_INPUT_HANDLE)
