@@ -8,6 +8,7 @@ Tests for various parts of L{twisted.web}.
 import os
 import zlib
 from io import BytesIO
+from typing import List
 
 from zope.interface import implementer
 from zope.interface.verify import verifyObject
@@ -15,12 +16,15 @@ from zope.interface.verify import verifyObject
 from twisted.internet import interfaces
 from twisted.internet.address import IPv4Address, IPv6Address
 from twisted.internet.task import Clock
+from twisted.internet.testing import EventLoggingObserver, StringTransport
 from twisted.logger import LogLevel, globalLogPublisher
 from twisted.python import failure, reflect
+from twisted.python.compat import iterbytes
 from twisted.python.filepath import FilePath
-from twisted.test.proto_helpers import EventLoggingObserver
 from twisted.trial import unittest
 from twisted.web import error, http, iweb, resource, server
+from twisted.web.resource import Resource
+from twisted.web.server import NOT_DONE_YET, Request, Site
 from twisted.web.static import Data
 from twisted.web.test.requesthelper import DummyChannel, DummyRequest
 from ._util import assertIsFilesystemTemporary
@@ -1153,9 +1157,10 @@ class NewRenderResource(resource.Resource):
 
 
 @implementer(resource.IResource)
-class HeadlessResource:
+class HeadlessWriter:
     """
-    A resource that implements GET but not HEAD.
+    A resource that implements GET but not HEAD, and
+    calls C{request.write()} when rendering GET.
     """
 
     allowedMethods = [b"GET"]
@@ -1171,21 +1176,46 @@ class HeadlessResource:
         return server.NOT_DONE_YET
 
     def isLeaf(self):
-        """
         # IResource.isLeaf
-        """
         raise NotImplementedError()
 
     def getChildWithDefault(self, name, request):
-        """
         # IResource.getChildWithDefault
-        """
         raise NotImplementedError()
 
     def putChild(self, path, child):
-        """
         # IResource.putChild
+        raise NotImplementedError()
+
+
+@implementer(resource.IResource)
+class HeadlessReturner:
+    """
+    A resource that implements GET but not HEAD, and
+    returns C{bytes} when rendering GET.
+    """
+
+    allowedMethods = [b"GET"]
+
+    def render(self, request):
         """
+        Leave the request open for future writes.
+        """
+        self.request = request
+        if request.method not in self.allowedMethods:
+            raise error.UnsupportedMethod(self.allowedMethods)
+        return b"some data"
+
+    def isLeaf(self):
+        # IResource.isLeaf
+        raise NotImplementedError()
+
+    def getChildWithDefault(self, name, request):
+        # IResource.getChildWithDefault
+        raise NotImplementedError()
+
+    def putChild(self, path, child):
+        # IResource.putChild
         raise NotImplementedError()
 
 
@@ -1253,21 +1283,44 @@ class NewRenderTests(unittest.TestCase):
         event = logObserver[0]
         self.assertEquals(event["log_level"], LogLevel.info)
 
-    def test_unsupportedHead(self):
+    def test_unsupportedHeadWrite(self):
+        """
+        HEAD requests against resource that only claim support for GET
+        should not include a body in the response. When the resource
+        returns C{NOT_DONE_YET} a cryptic message is logged and no
+        Content-Length header is synthesized.
+        """
+        logs = EventLoggingObserver.createWithCleanup(self, globalLogPublisher)
+
+        resource = HeadlessWriter()
+        req = self._getReq(resource)
+        req.requestReceived(b"HEAD", b"/newrender", b"HTTP/1.0")
+        headers, body = req.transport.written.getvalue().split(b"\r\n\r\n")
+
+        self.assertEqual(req.code, 200)
+        self.assertEqual(body, b"")
+        self.assertNotIn(b"Content-Length:", headers)
+        [faking, cryptic] = logs
+        self.assertIn("Using GET to fake a HEAD request", faking["log_format"])
+        self.assertIn("it got away from me", cryptic["log_format"])
+
+    def test_unsupportedHeadReturn(self):
         """
         HEAD requests against resource that only claim support for GET
         should not include a body in the response.
         """
-        logObserver = EventLoggingObserver.createWithCleanup(self, globalLogPublisher)
+        logs = EventLoggingObserver.createWithCleanup(self, globalLogPublisher)
 
-        resource = HeadlessResource()
+        resource = HeadlessReturner()
         req = self._getReq(resource)
         req.requestReceived(b"HEAD", b"/newrender", b"HTTP/1.0")
         headers, body = req.transport.written.getvalue().split(b"\r\n\r\n")
+
         self.assertEqual(req.code, 200)
         self.assertEqual(body, b"")
-
-        self.assertEquals(2, len(logObserver))
+        self.assertIn(b"Content-Length:", headers)
+        [faking] = logs
+        self.assertIn("Using GET to fake a HEAD request", faking["log_format"])
 
     def test_noBytesResult(self):
         """
@@ -1779,53 +1832,6 @@ class LogEscapingTests(unittest.TestCase):
         )
 
 
-class ServerAttributesTests(unittest.TestCase):
-    """
-    Tests that deprecated twisted.web.server attributes raise the appropriate
-    deprecation warnings when used.
-    """
-
-    def test_deprecatedAttributeDateTimeString(self):
-        """
-        twisted.web.server.date_time_string should not be used; instead use
-        twisted.web.http.datetimeToString directly
-        """
-        server.date_time_string
-        warnings = self.flushWarnings(
-            offendingFunctions=[self.test_deprecatedAttributeDateTimeString]
-        )
-
-        self.assertEqual(len(warnings), 1)
-        self.assertEqual(warnings[0]["category"], DeprecationWarning)
-        self.assertEqual(
-            warnings[0]["message"],
-            (
-                "twisted.web.server.date_time_string was deprecated in Twisted "
-                "12.1.0: Please use twisted.web.http.datetimeToString instead"
-            ),
-        )
-
-    def test_deprecatedAttributeStringDateTime(self):
-        """
-        twisted.web.server.string_date_time should not be used; instead use
-        twisted.web.http.stringToDatetime directly
-        """
-        server.string_date_time
-        warnings = self.flushWarnings(
-            offendingFunctions=[self.test_deprecatedAttributeStringDateTime]
-        )
-
-        self.assertEqual(len(warnings), 1)
-        self.assertEqual(warnings[0]["category"], DeprecationWarning)
-        self.assertEqual(
-            warnings[0]["message"],
-            (
-                "twisted.web.server.string_date_time was deprecated in Twisted "
-                "12.1.0: Please use twisted.web.http.stringToDatetime instead"
-            ),
-        )
-
-
 class ExplicitHTTPFactoryReactor(unittest.TestCase):
     """
     L{http.HTTPFactory} accepts explicit reactor selection.
@@ -1849,3 +1855,78 @@ class ExplicitHTTPFactoryReactor(unittest.TestCase):
 
         factory = http.HTTPFactory()
         self.assertIs(factory.reactor, reactor)
+
+
+class QueueResource(Resource):
+    """
+    Add all requests to an internal queue,
+    without responding to the requests.
+    You can access the requests from the queue and handle their response.
+    """
+
+    isLeaf = True
+
+    def __init__(self) -> None:
+        super().__init__()
+        self.dispatchedRequests: List[Request] = []
+
+    def render_GET(self, request: Request) -> int:
+        self.dispatchedRequests.append(request)
+        return NOT_DONE_YET
+
+
+class TestRFC9112Section932(unittest.TestCase):
+    """
+    Verify that HTTP/1.1 request ordering is preserved.
+    """
+
+    def test_multipleRequestsInOneSegment(self) -> None:
+        """
+        Twisted MUST NOT respond to a second HTTP/1.1 request while the first
+        is still pending.
+        """
+        qr = QueueResource()
+        site = Site(qr)
+        proto = site.buildProtocol(None)
+        serverTransport = StringTransport()
+        proto.makeConnection(serverTransport)
+        proto.dataReceived(
+            b"GET /first HTTP/1.1\r\nHost: a\r\n\r\n"
+            b"GET /second HTTP/1.1\r\nHost: a\r\n\r\n"
+        )
+        # The TCP data contains 2 requests,
+        # but only 1 request was dispatched,
+        # as the first request was not yet finalized.
+        self.assertEqual(len(qr.dispatchedRequests), 1)
+        # The first request is finalized and the
+        # second request is dispatched right away.
+        qr.dispatchedRequests[0].finish()
+        self.assertEqual(len(qr.dispatchedRequests), 2)
+
+    def test_multipleRequestsInDifferentSegments(self) -> None:
+        """
+        Twisted MUST NOT respond to a second HTTP/1.1 request while the first
+        is still pending, even if the second request is received in a separate
+        TCP package.
+        """
+        qr = QueueResource()
+        site = Site(qr)
+        proto = site.buildProtocol(None)
+        serverTransport = StringTransport()
+        proto.makeConnection(serverTransport)
+        raw_data = (
+            b"GET /first HTTP/1.1\r\nHost: a\r\n\r\n"
+            b"GET /second HTTP/1.1\r\nHost: a\r\n\r\n"
+        )
+        # Just go byte by byte for the extreme case in which each byte is
+        # received in a separate TCP package.
+        for chunk in iterbytes(raw_data):
+            proto.dataReceived(chunk)
+        # The TCP data contains 2 requests,
+        # but only 1 request was dispatched,
+        # as the first request was not yet finalized.
+        self.assertEqual(len(qr.dispatchedRequests), 1)
+        # The first request is finalized and the
+        # second request is dispatched right away.
+        qr.dispatchedRequests[0].finish()
+        self.assertEqual(len(qr.dispatchedRequests), 2)
